package javax.sql.rowset.serial;

import java.sql.SQLXML;
import java.sql.RowId;
import java.sql.NClob;
import java.net.URL;
import java.sql.Array;
import java.sql.Struct;
import java.sql.Clob;
import java.sql.Blob;
import java.sql.Ref;
import java.sql.SQLData;
import java.io.InputStreamReader;
import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.Reader;
import java.sql.Timestamp;
import java.sql.Time;
import java.sql.Date;
import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.Map;
import java.util.Vector;
import java.sql.SQLOutput;

public class SQLOutputImpl implements SQLOutput
{
    private Vector attribs;
    private Map map;
    
    public SQLOutputImpl(Vector<?> attribs, Map<String, ?> map) throws SQLException {
        if (attribs == null || map == null) {
            throw new SQLException("Cannot instantiate a SQLOutputImpl instance with null parameters");
        }
        this.attribs = attribs;
        this.map = map;
        return;
    }
    
    @Override
    public void writeString(String e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeBoolean(boolean b) throws SQLException {
        this.attribs.add(Boolean.valueOf(b));
    }
    
    @Override
    public void writeByte(byte b) throws SQLException {
        this.attribs.add(Byte.valueOf(b));
    }
    
    @Override
    public void writeShort(short s) throws SQLException {
        this.attribs.add(Short.valueOf(s));
    }
    
    @Override
    public void writeInt(int i) throws SQLException {
        this.attribs.add(Integer.valueOf(i));
    }
    
    @Override
    public void writeLong(long l) throws SQLException {
        this.attribs.add(Long.valueOf(l));
    }
    
    @Override
    public void writeFloat(float f) throws SQLException {
        this.attribs.add(Float.valueOf(f));
    }
    
    @Override
    public void writeDouble(double d) throws SQLException {
        this.attribs.add(Double.valueOf(d));
    }
    
    @Override
    public void writeBigDecimal(BigDecimal e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeBytes(byte[] e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeDate(Date e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeTime(Time e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeTimestamp(Timestamp e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeCharacterStream(Reader in) throws SQLException {
        BufferedReader bufferedReader = new BufferedReader(in);
        try {
            int read;
            while ((read = bufferedReader.read()) != -1) {
                int c = (char)read;
                StringBuffer buffer = new StringBuffer();
                buffer.append((char)c);
                this.writeString(new String(buffer).concat(bufferedReader.readLine()));
            }
        }
        catch (IOException ex) {}
    }
    
    @Override
    public void writeAsciiStream(InputStream in) throws SQLException {
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(in));
        try {
            int read;
            while ((read = bufferedReader.read()) != -1) {
                int c = (char)read;
                StringBuffer buffer = new StringBuffer();
                buffer.append((char)c);
                this.writeString(new String(buffer).concat(bufferedReader.readLine()));
            }
        }
        catch (IOException ex) {
            throw new SQLException(ex.getMessage());
        }
    }
    
    @Override
    public void writeBinaryStream(InputStream in) throws SQLException {
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(in));
        try {
            int read;
            while ((read = bufferedReader.read()) != -1) {
                int c = (char)read;
                StringBuffer buffer = new StringBuffer();
                buffer.append((char)c);
                this.writeString(new String(buffer).concat(bufferedReader.readLine()));
            }
        }
        catch (IOException ex) {
            throw new SQLException(ex.getMessage());
        }
    }
    
    @Override
    public void writeObject(SQLData sqlData) throws SQLException {
        if (sqlData == null) {
            this.attribs.add(null);
        }
        else {
            this.attribs.add(new SerialStruct(sqlData, this.map));
        }
    }
    
    @Override
    public void writeRef(Ref ref) throws SQLException {
        if (ref == null) {
            this.attribs.add(null);
        }
        else {
            this.attribs.add(new SerialRef(ref));
        }
    }
    
    @Override
    public void writeBlob(Blob blob) throws SQLException {
        if (blob == null) {
            this.attribs.add(null);
        }
        else {
            this.attribs.add(new SerialBlob(blob));
        }
    }
    
    @Override
    public void writeClob(Clob clob) throws SQLException {
        if (clob == null) {
            this.attribs.add(null);
        }
        else {
            this.attribs.add(new SerialClob(clob));
        }
    }
    
    @Override
    public void writeStruct(Struct struct) throws SQLException {
        this.attribs.add(new SerialStruct(struct, this.map));
    }
    
    @Override
    public void writeArray(Array array) throws SQLException {
        if (array == null) {
            this.attribs.add(null);
        }
        else {
            this.attribs.add(new SerialArray(array, this.map));
        }
    }
    
    @Override
    public void writeURL(URL url) throws SQLException {
        if (url == null) {
            this.attribs.add(null);
        }
        else {
            this.attribs.add(new SerialDatalink(url));
        }
    }
    
    @Override
    public void writeNString(String e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeNClob(NClob e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeRowId(RowId e) throws SQLException {
        this.attribs.add(e);
    }
    
    @Override
    public void writeSQLXML(SQLXML e) throws SQLException {
        this.attribs.add(e);
    }
}
